﻿* Encoding: UTF-8.
*THE FOLLOWING SYNTAX WAS DEVELOPED FROM SAS CODE PUBLISHED BY 
LARRY R. PRICE IN THE ARTICLE 'EQUIPERECENT: A SAS PROGRAM FOR EQUIVALENT SCORES'
APPLIED PSYCHOLOGICAL MEASURMENT 2001.

*the data file should contain the records for N number of examinees who took both forms of the test -
the variables (test scores) can be in any order in this file.
DEFINE INFILE ()
'C:\temp\ART and Fall 08.sav'
!ENDDEFINE.

*include the variable name for VARX (variable X), the minimum & maximum values for VARX, 
and the increment of measurement for VARX below.
DEFINE VARX () ALGEBRA_READINESS !ENDDEFINE.
DEFINE MINX () 7 !ENDDEFINE.
DEFINE MAXX () 49 !ENDDEFINE.
DEFINE INCX () 1 !ENDDEFINE.

*include the variable name for VARY (variable Y), the minimum & maximum values for VARY, 
and the increment of measurement for VARY below.
DEFINE VARY () FALL_08_TestRITScore !ENDDEFINE.
DEFINE MINY () 190 !ENDDEFINE.
DEFINE MAXY () 255 !ENDDEFINE.
DEFINE INCY () 1 !ENDDEFINE.

GET
  FILE=INFILE.
DATASET NAME IN WINDOW=FRONT.

*first, the percent of examinees and cumulative percent is determined for each value of of the VARX.
RANK VARIABLES=VARX (A)
  /PERCENT  INTO CUMPCTX
  /PRINT=YES
  /TIES=HIGH.

SORT CASES BY VARX.
AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /PRESORTED
  /BREAK=VARX
  /COUNTX=N.

AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /PRESORTED
  /BREAK=
  /TOTALX=NU(VARX).

COMPUTE PCTX=(COUNTX / TOTALX) * 100.
COMPUTE PCTRKX = (PCTX * INCX/2) + (CUMPCTX - PCTX).
EXECUTE.

* Any duplicate cases of each value of VARX is removed here.
SORT CASES BY VARX(A).
MATCH FILES
  /FILE=*
  /BY VARX
  /FIRST=PrimaryFirst.
VARIABLE LABELS  PrimaryFirst 'Indicator of each first matching case as Primary'.
VALUE LABELS  PrimaryFirst 0 'Duplicate Case' 1 'Primary Case'.
VARIABLE LEVEL  PrimaryFirst (ORDINAL).
FILTER  BY PrimaryFirst.
EXECUTE.
FILTER OFF.
USE ALL.
SELECT IF (PrimaryFirst = 1).
EXECUTE.

SAVE OUTFILE='C:\TEMP\OUTX.sav'
  /DROP=VARY COUNTX TOTALX PrimaryFirst
  /COMPRESSED.

*the following input program creates a list of all possible values for VARX, based on the min and max values
since the actual data file may not include cases for each possible value.
INPUT PROGRAM.
LOOP #I=MINX TO MAXX.
COMPUTE VARX=(#I+INCX)-1.
END CASE.
END LOOP.
END FILE.
END INPUT PROGRAM.
FORMATS VARX (F4.0).
SAVE OUTFILE='C:\Temp\OUTX_LIST.SAV'.

*NEXT WE WILL GET THE OUTX FILE, THEN MERGE THE DATA FROM OUTX INTO THE OUTX_LIST FILE.
GET FILE='C:\Temp\OUTX_LIST.sav'.
DATASET NAME OUTX_LIST WINDOW=FRONT.

MATCH FILES /FILE=*
  /FILE='C:\Temp\OUTX.sav'
  /BY VARX.
EXECUTE.

*the following will find any missing values and replace those with zeros.
IF  (MISSING(PCTX) = 1) PCTX=0.
EXECUTE.

*then, for any cumulative percent that is missing, the value from the previous record is copied down.
*NOTE: if your file has more than 5 records in a row that has missing values, you can duplicate the code below
to reflect more "lag" variables.
SHIFT VALUES VARIABLE=CUMPCTX RESULT=PREV1_CUMPCTX LAG=1
/VARIABLE=CUMPCTX RESULT=PREV2_CUMPCTX LAG=2
/VARIABLE=CUMPCTX RESULT=PREV3_CUMPCTX LAG=3
/VARIABLE=CUMPCTX RESULT=PREV4_CUMPCTX LAG=4
/VARIABLE=CUMPCTX RESULT=PREV5_CUMPCTX LAG=5.

IF (MISSING(CUMPCTX)=1) CUMPCTX=PREV1_CUMPCTX.
EXECUTE.
IF (MISSING(CUMPCTX)=1) CUMPCTX=PREV2_CUMPCTX.
EXECUTE.
IF (MISSING(CUMPCTX)=1) CUMPCTX=PREV3_CUMPCTX.
EXECUTE.
IF (MISSING(CUMPCTX)=1) CUMPCTX=PREV4_CUMPCTX.
EXECUTE.
IF (MISSING(CUMPCTX)=1) CUMPCTX=PREV5_CUMPCTX.
EXECUTE.

COMPUTE PCTRKX = (PCTX * INCX/2) + (CUMPCTX - PCTX).
EXECUTE.

*NOTE: if you added more "lag" variables above, be sure to add those variable names to the DROP line below.
SAVE OUTFILE='C:\Temp\OUTX.SAV'
/DROP PREV1_CUMPCTX PREV2_CUMPCTX PREV3_CUMPCTX PREV4_CUMPCTX PREV5_CUMPCTX .

DATASET CLOSE OUTX_LIST.

*******************************************************************
END OF SYNTAX FOR VAR X.
*******************************************************************
*THE FOLLOWING IS WORKING WITH THE VAR Y DATA.

GET
  FILE=INFILE.
DATASET NAME IN WINDOW=FRONT.

RANK VARIABLES=VARY (A)
  /PERCENT  INTO CUMPCTY
  /PRINT=YES
  /TIES=HIGH.

SORT CASES BY VARY.
AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /PRESORTED
  /BREAK=VARY
  /COUNTY=N.

AGGREGATE
  /OUTFILE=* MODE=ADDVARIABLES
  /PRESORTED
  /BREAK=
  /TOTALY=NU(VARY).

COMPUTE PCTY=(COUNTY / TOTALY) * 100.
EXECUTE.

SORT CASES BY VARY(A).
MATCH FILES
  /FILE=*
  /BY VARY
  /FIRST=PrimaryFirst.
VARIABLE LABELS  PrimaryFirst 'Indicator of each first matching case as Primary'.
VALUE LABELS  PrimaryFirst 0 'Duplicate Case' 1 'Primary Case'.
VARIABLE LEVEL  PrimaryFirst (ORDINAL).
FILTER  BY PrimaryFirst.
EXECUTE.
FILTER OFF.
USE ALL.
SELECT IF (PrimaryFirst = 1).
EXECUTE.

SAVE OUTFILE='C:\TEMP\OUTY.sav'
  /DROP=VARX COUNTY TOTALY PrimaryFirst
  /COMPRESSED.

INPUT PROGRAM.
LOOP #I=MINY TO MAXY.
COMPUTE VARY=(#I+INCY)-1.
END CASE.
END LOOP.
END FILE.
END INPUT PROGRAM.
FORMATS VARY (F4.0).
SAVE OUTFILE='C:\Temp\OUTY_LIST.SAV'.

GET FILE='C:\Temp\OUTY_LIST.sav'.
DATASET NAME OUTY_LIST WINDOW=FRONT.

MATCH FILES /FILE=*
  /FILE='C:\Temp\OUTY.sav'
  /BY VARY.
EXECUTE.

IF  (MISSING(PCTY) = 1) PCTY=0.
EXECUTE.

*NOTE: if your file has more than 5 records in a row that has missing values, you can duplicate the code below
to reflect more "lag" variables.
SHIFT VALUES VARIABLE=CUMPCTY RESULT=PREV1_CUMPCTY LAG=1
/VARIABLE=CUMPCTY RESULT=PREV2_CUMPCTY LAG=2
/VARIABLE=CUMPCTY RESULT=PREV3_CUMPCTY LAG=3
/VARIABLE=CUMPCTY RESULT=PREV4_CUMPCTY LAG=4
/VARIABLE=CUMPCTY RESULT=PREV5_CUMPCTY LAG=5.

IF (MISSING(CUMPCTY)=1) CUMPCTY=PREV1_CUMPCTY.
EXECUTE.
IF (MISSING(CUMPCTY)=1) CUMPCTY=PREV2_CUMPCTY.
EXECUTE.
IF (MISSING(CUMPCTY)=1) CUMPCTY=PREV3_CUMPCTY.
EXECUTE.
IF (MISSING(CUMPCTY)=1) CUMPCTY=PREV4_CUMPCTY.
EXECUTE.
IF (MISSING(CUMPCTY)=1) CUMPCTY=PREV5_CUMPCTY.
EXECUTE.

*NOTE: if you added more "lag" variables above, be sure to add those variable names to the DROP line below.
SAVE OUTFILE='C:\Temp\OUTY.SAV'
/DROP PREV1_CUMPCTY PREV2_CUMPCTY PREV3_CUMPCTY PREV4_CUMPCTY PREV5_CUMPCTY .

DATASET CLOSE OUTY_LIST.
DATASET CLOSE IN.

*******************************************************************
END OF SYNTAX FOR VAR Y.
*******************************************************************

*PREPPING THE DATA FOR THE COMBINATION MACRO.
GET FILE='C:\TEMP\OUTY_LIST.SAV'
/KEEP VARY.
DATASET NAME OUTY_LIST.
GET FILE='C:\TEMP\OUTX_LIST.SAV'
/KEEP VARX.
DATASET NAME OUTX_LIST WINDOW=FRONT.

MATCH FILES /FILE=*
  /FILE='OUTY_LIST'.
EXECUTE.

SAVE TRANSLATE OUTFILE='C:\Temp\OUT_COMB.csv'
  /TYPE=CSV
  /MAP
  /REPLACE
  /CELLS=VALUES.

DATASET CLOSE OUTX_LIST.
DATASET CLOSE OUTY_LIST.

*/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
THE FOLLOWING IS THE MACRO FOR COMBINATIONS
             finding all possible combinations of the OUTX and OUTY files
this syntax was taken from http://www.spsstools.net/ and modified to produce the result shown below
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////.
*This macro creates two new variables with the combination of all
pairs of categories for the variables X and Z.

*The result is two variables (var001 and var002) with cases that have all possible combinations of X and Z

*var 001	var002
1	1
1	2
1	3
2	1
2	2
2	3
3	1
3	2
3	3
.

SET MPRINT=no.
*////////////////////.
define !compint (!pos !tokens(1)
             /!pos !tokens(1))

compute nobreak = 1.

aggregate outfile ='c:\temp\SPSS_002.tmp'
     /break= nobreak
     /minv1 = min(!1)
     /maxv1 = max(!1)
     /minv2 = min(!2)
     /maxv2 = max(!2).

match files /file=* /table= 'c:\temp\SPSS_002.tmp'
     /by= nobreak.

do if ($casenum=1).
write outfile= 'c:\temp\minmax.sps'
          /'define min1() ' minv1 ' !enddefine.'
          /'define max1() ' maxv1 ' !enddefine.'
          /'define min2() ' minv2 ' !enddefine.'
          /'define max2() ' maxv2 ' !enddefine.'.
end if.
execute.

include file='c:\temp\minmax.sps'.
EXECUTE.
!compin2 !1 !2.
!enddefine.
*////////////////////.


*////////////////////.
define !compin2(!pos=!tokens(1) /!pos=!tokens(1))
!do !i = !EVAL(min1) !to !EVAL(max1)
     !do !j = !EVAL(min2) !to !EVAL(max2)
          !let !varnam = !concat(i,!1,!2,!i,!j)
          numeric !varnam (f5).
	DO IF X = minv1.
	COMPUTE !varnam = !i.
	ELSE.
	COMPUTE !varnam = !j.
	END IF.
     !doend.
exec.
!doend.

FILTER OFF.
USE ALL.
SELECT IF (X = minv1 | X = maxv1).
EXECUTE.

match files /file=* /drop= nobreak minv1 minv2 maxv1 maxv2.
exec.
!enddefine.
*////////////////////.

data list FILE='C:\TEMP\OUT_COMB.CSV' FREE / X Z.

FORMATS X Z (F3.0).
SET MPRINT=yes.
!compint X Z.

SAVE OUTFILE='C:\TEMP\COMBOS.sav'
/DROP X Z.

GET FILE='C:\TEMP\COMBOS.sav'.
DATASET NAME COMBOS WINDOW=FRONT.

FLIP VARIABLES=ALL.
DATASET NAME COMBOS_FLIPPED WINDOW=FRONT.

DATASET CLOSE COMBOS.

SAVE OUTFILE='C:\TEMP\COMBOS.sav'
/DROP CASE_LBL.
DATASET CLOSE COMBOS_FLIPPED.

*/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
END OF THE COMBINATIONS MACROS
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////.

*THE FOLLOWING CODE MERGES THE DATA FROM THE OUTX AND OUTY FILES
INTO THE COMBINATIONS LIST.
GET FILE='C:\Temp\OUTX.sav'.
DATASET NAME OUTX.
GET FILE='C:\Temp\OUTY.sav'.
DATASET NAME OUTY.
GET FILE='C:\Temp\COMBOS.sav'.
DATASET NAME COMBOS WINDOW=FRONT.

RENAME VARIABLES (var001=VARX).
RENAME VARIABLES (var002=VARY).

*THE FOLLOWING CODE COPIES THE VARIABLE PROPERTIES OF THE "VARX" variable in the OUTX data file
to the 'var001' variable in the COMBO data file.

APPLY DICTIONARY
  /FROM OUTX
  /SOURCE VARIABLES=VARX
  /TARGET VARIABLES=VARX
  /FILEINFO 
  /VARINFO ALL.    

*AND THEN COPIES THE VARIABLE PROPERTIES OF THE "VARY" variable in the OUTY data file
to the 'var002' variable in the COMBO data file.

APPLY DICTIONARY
  /FROM OUTY
  /SOURCE VARIABLES=VARY
  /TARGET VARIABLES=VARY
  /FILEINFO
  /VARINFO ALL.    

DATASET ACTIVATE COMBOS.
SORT CASES BY VARX (A).
EXECUTE.

MATCH FILES /FILE=*
  /TABLE='OUTX'
  /BY VARX.
EXECUTE.

DATASET ACTIVATE COMBOS.
SORT CASES BY VARY (A).
EXECUTE.

MATCH FILES /FILE=*
  /TABLE='OUTY'
  /BY VARY.
EXECUTE.

DATASET CLOSE OUTX.
DATASET CLOSE OUTY.


*******THE FOLLOWING NOTE IS QUOTED DIRECTLY FROM LARRY PRICE'S SAS CODE****
******** This section provides a basis for incorporating 
******** Kolen's (1995, p.44) formula that provides the upper *****
******** and lower boundaries relative to the distribution ********
******** of Y values. *********************************************
******** This step results in two output data sets - one **********
******** for upper and one for lower boundaries of Y **************
*******************************************************************

**********Kolen's 2004 edition, p. 45*****************************


DATASET ACTIVATE COMBOS.
COMPUTE YU = INCY*((PCTRKX - (CUMPCTY - PCTY)) / PCTY) + (VARY - .5 * INCY).
COMPUTE YL = INCY * ((PCTRKX - (CUMPCTY - PCTY)) / PCTY) + (VARY - .5 * INCY).
VARIABLE WIDTH YU YL (20).
FORMATS YU (F5.10) / YL (F5.10).
EXECUTE.

DATASET COPY  RESULTU.
DATASET ACTIVATE  RESULTU.
FILTER OFF.
USE ALL.
SELECT IF (((CUMPCTY - PCTY) <= PCTRKX) & (CUMPCTY > PCTRKX)).
EXECUTE.

DATASET ACTIVATE  COMBOS.
DATASET COPY  RESULTL.
DATASET ACTIVATE  RESULTL.
FILTER OFF.
USE ALL.
SELECT IF ((CUMPCTY >= PCTRKX) & ((CUMPCTY - PCTY) < PCTRKX)).
EXECUTE.

DATASET ACTIVATE  RESULTU.
MATCH FILES /FILE=*
  /FILE='RESULTL'
  /BY VARX.
EXECUTE.

COMPUTE YX = (YL + YU) / 2.
COMPUTE X = VARX.
COMPUTE YX2=RND(YX,INCY).
EXECUTE.


SAVE OUTFILE='C:\TEMP\FINAL_EQUIPERCENTILE.sav'
/KEEP=X YX2 YL YU.

DATASET CLOSE RESULTL.
DATASET CLOSE RESULTU.
DATASET CLOSE COMBOS.

GET FILE='C:\TEMP\FINAL_EQUIPERCENTILE.sav'.
DATASET NAME FINAL WINDOW=FRONT.
RENAME VARIABLES (YX2=YX).

